import { __awaiter, __rest } from 'tslib';
import * as i0 from '@angular/core';
import { EventEmitter, Directive, Output, Input, HostListener, Injectable, NgModule } from '@angular/core';
import * as i1 from '@angular/common/http';
import { HttpRequest, HttpHeaders, HttpParams, HttpHeaderResponse, HttpEventType } from '@angular/common/http';
import { throwError } from 'rxjs';
import { filter, map, catchError } from 'rxjs/operators';

/**
 * get files with structure
 * @param files source files
 * @returns structure file
 */
function getStructureFiles(files) {
    return files.reduce((acc, curr) => {
        const { webkitRelativePath, filepath } = curr;
        const currentFullPath = filepath || webkitRelativePath;
        if (currentFullPath) {
            const paths = currentFullPath.split('/');
            // remove file name
            paths.pop();
            const tmp = [...paths];
            let index = 0;
            let currParent = acc;
            while (tmp.length > 0) {
                const name = tmp.shift();
                index++;
                let currP = currParent.find((x) => x.name === name);
                if (!currP) {
                    currP = {
                        name: name,
                        path: paths.slice(0, index).join('/'),
                        children: [],
                        isDirectory: true,
                    };
                    currParent.push(currP);
                }
                currParent = currP.children;
                if (tmp.length === 0) {
                    currParent.push(curr);
                }
            }
        }
        return acc;
    }, []);
}

function getUploadResult(files, accept, multiple, option, structure) {
    let result;
    if (multiple !== undefined) {
        result = checkAllFile(files, accept, option);
    }
    else if (files.length === 1) {
        result = checkAllFile(files[0], accept, option);
    }
    else {
        result = 0 /* NumError */;
    }
    if (structure === 'directory' && result instanceof Array) {
        return getStructureFiles(result);
    }
    return result;
}
function checkAllFile(file, accept, option) {
    if (file instanceof FileList || file instanceof Array) {
        const files = [];
        let err;
        const allPass = Array.from(file).every((f) => {
            const error = checkOneFile(f, accept, option);
            const noError = error instanceof File;
            err = error;
            if (noError) {
                files.push(f);
            }
            // if skipInvalid, all pass
            return option.skipInvalid || noError;
        });
        if (!allPass) {
            return err;
        }
        return files;
    }
    return checkOneFile(file, accept, option);
}
function checkOneFile(file, accept, option) {
    if (!checkFileType(file, accept)) {
        return 1 /* TypeError */;
    }
    if (!checkFileSize(file, option)) {
        return 2 /* SizeError */;
    }
    return file;
}
function checkFileType(file, accept) {
    if (accept) {
        const acceptedFilesArray = accept.split(',');
        return acceptedFilesArray.some((type) => {
            const validType = type.trim();
            if (validType.charAt(0) === '.') {
                return file.name.toLowerCase().endsWith(validType.toLowerCase());
            }
            else if (/\/\*$/.test(validType)) {
                // This is something like a image/* mime type
                return (file.type.replace(/\/.*$/, '') === validType.replace(/\/.*$/, ''));
            }
            return file.type === validType;
        });
    }
    return true;
}
function checkFileSize(file, option) {
    if (option) {
        const size = file.size;
        const chkSize = option.size;
        if (chkSize &&
            ((chkSize.min && size < chkSize.min) ||
                (chkSize.max && size > chkSize.max))) {
            return false;
        }
    }
    return true;
}

const getFilesFromEntry = (entry, path = '') => __awaiter(void 0, void 0, void 0, function* () {
    return entry.isDirectory
        ? yield readDir(entry, path)
        : [yield readFile(entry, path)];
});
const readFile = (entry, path = '') => {
    return new Promise((resolve, reject) => {
        entry.file((file) => {
            file.filepath = path + file.name;
            resolve(file);
        }, reject);
    });
};
const readDir = (entry, path) => __awaiter(void 0, void 0, void 0, function* () {
    const dirReader = entry.createReader();
    const newPath = `${path + entry.name}/`;
    let files = [];
    let newFiles;
    do {
        newFiles = yield dirReadEntries(dirReader, newPath);
        files = files.concat(newFiles);
    } while (newFiles.length > 0);
    return files;
});
const dirReadEntries = (dirReader, path) => {
    return new Promise((resolve, reject) => {
        dirReader.readEntries((entries) => __awaiter(void 0, void 0, void 0, function* () {
            let files = [];
            for (let entry of entries) {
                const itemFiles = yield getFilesFromEntry(entry, path);
                files = files.concat(itemFiles);
            }
            resolve(files);
        }), reject);
    });
};
/** get files from `webkitGetAsEntry` */
const getFilesFromGetAsEntry = (e) => __awaiter(void 0, void 0, void 0, function* () {
    let files = [];
    const { items } = e.dataTransfer;
    for (let i = 0; i < items.length; i++) {
        const entry = items[i].webkitGetAsEntry();
        const result = yield getFilesFromEntry(entry);
        files.push(...result);
    }
    return files;
});

const stopEvent = (e) => {
    e.stopPropagation();
    e.preventDefault();
};
/**
 * provide a directive for you to set area can be drop file into
 */
class NgxfDropDirective {
    constructor(_elm, _render) {
        this._elm = _elm;
        this._render = _render;
        /** when get drop file into area that will trigger */
        this.onUpload = new EventEmitter();
        this.fileOption = {};
        /**
         * add class when drop into this drop area
         * @default 'drop'
         */
        this.dropClass = 'drop';
    }
    drop(e) {
        return __awaiter(this, void 0, void 0, function* () {
            stopEvent(e);
            this._render.removeClass(this._elm.nativeElement, this.dropClass);
            let files;
            try {
                files = yield getFilesFromGetAsEntry(e);
            }
            catch (error) {
                files = e.dataTransfer.files;
            }
            const result = getUploadResult(files, this.accept, this.multiple, this.fileOption, this.structure);
            this.onUpload.emit(result);
        });
    }
    dragenter(e) {
        stopEvent(e);
        this._render.addClass(this._elm.nativeElement, this.dropClass);
    }
    dragleave(e) {
        stopEvent(e);
        this._render.removeClass(this._elm.nativeElement, this.dropClass);
    }
}
NgxfDropDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfDropDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Directive });
NgxfDropDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.0.1", type: NgxfDropDirective, selector: "[ngxf-drop]", inputs: { fileOption: ["ngxf-validate", "fileOption"], multiple: "multiple", accept: "accept", structure: "structure", dropClass: ["drop-class", "dropClass"] }, outputs: { onUpload: "ngxf-drop" }, host: { listeners: { "drop": "drop($event)", "dragover": "dragenter($event)", "dragenter": "dragenter($event)", "dragleave": "dragleave($event)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfDropDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxf-drop]',
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }]; }, propDecorators: { onUpload: [{
                type: Output,
                args: ['ngxf-drop']
            }], fileOption: [{
                type: Input,
                args: ['ngxf-validate']
            }], multiple: [{
                type: Input
            }], accept: [{
                type: Input
            }], structure: [{
                type: Input
            }], dropClass: [{
                type: Input,
                args: ['drop-class']
            }], drop: [{
                type: HostListener,
                args: ['drop', ['$event']]
            }], dragenter: [{
                type: HostListener,
                args: ['dragover', ['$event']]
            }, {
                type: HostListener,
                args: ['dragenter', ['$event']]
            }], dragleave: [{
                type: HostListener,
                args: ['dragleave', ['$event']]
            }] } });

/**
 * provide a directive that can let you select file upload by click
 */
class NgxfSelectDirective {
    constructor(_render, _elm) {
        this._render = _render;
        this._elm = _elm;
        /** when use select some files end, that will trigger */
        this.uploadOutput = new EventEmitter();
        this.fileOption = {};
    }
    click() {
        this.bindBeforeClick();
        this.fileElm.click();
    }
    ngAfterViewInit() {
        this.fileElm = this._render.createElement('input');
        this._render.setAttribute(this.fileElm, 'type', 'file');
        this._render.setStyle(this.fileElm, 'display', 'none');
        if (this.folder !== undefined && this.multiple !== undefined) {
            this._render.setAttribute(this.fileElm, 'webkitdirectory', '');
        }
        this._render.appendChild(this._elm.nativeElement.parentNode, this.fileElm);
    }
    ngOnDestroy() {
        var _a;
        (_a = this.changeListen) === null || _a === void 0 ? void 0 : _a.call(this);
        this._render.removeChild(this._elm.nativeElement.parentNode, this.fileElm);
    }
    // because bind before click, that can change input to change accept and validate
    bindBeforeClick() {
        var _a;
        if (this.multiple !== undefined) {
            this._render.setAttribute(this.fileElm, 'multiple', '');
        }
        else {
            this._render.removeAttribute(this.fileElm, 'multiple');
        }
        this._render.setAttribute(this.fileElm, 'accept', this.accept);
        (_a = this.changeListen) === null || _a === void 0 ? void 0 : _a.call(this);
        this.changeListen = this._render.listen(this.fileElm, 'change', (e) => {
            var _a, _b, _c;
            // when length is more than 0
            if ((_b = (_a = this.fileElm) === null || _a === void 0 ? void 0 : _a.files) === null || _b === void 0 ? void 0 : _b.length) {
                const result = getUploadResult(this.fileElm.files, this.accept, this.multiple, this.fileOption, this.structure);
                this.uploadOutput.emit(result);
            }
            this.fileElm.value = '';
            (_c = this.changeListen) === null || _c === void 0 ? void 0 : _c.call(this);
        });
    }
}
NgxfSelectDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfSelectDirective, deps: [{ token: i0.Renderer2 }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
NgxfSelectDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.0.1", type: NgxfSelectDirective, selector: "[ngxf-select]", inputs: { fileOption: ["ngxf-validate", "fileOption"], multiple: "multiple", accept: "accept", folder: "folder", structure: "structure" }, outputs: { uploadOutput: "ngxf-select" }, host: { listeners: { "click": "click()" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfSelectDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxf-select]',
                }]
        }], ctorParameters: function () { return [{ type: i0.Renderer2 }, { type: i0.ElementRef }]; }, propDecorators: { uploadOutput: [{
                type: Output,
                args: ['ngxf-select']
            }], fileOption: [{
                type: Input,
                args: ['ngxf-validate']
            }], multiple: [{
                type: Input
            }], accept: [{
                type: Input
            }], folder: [{
                type: Input
            }], structure: [{
                type: Input
            }], click: [{
                type: HostListener,
                args: ['click']
            }] } });

/**
 * provide a directive for you to set area can be parse file into
 */
class NgxfParseDirective {
    constructor() {
        /** when get parse file that will trigger */
        this.uploadOutput = new EventEmitter();
        this.fileOption = {};
    }
    parse(event) {
        const clipboardData = event.clipboardData;
        if ((clipboardData === null || clipboardData === void 0 ? void 0 : clipboardData.files.length) === 0)
            return;
        const result = getUploadResult(clipboardData === null || clipboardData === void 0 ? void 0 : clipboardData.files, this.accept, this.multiple, this.fileOption, this.structure);
        this.uploadOutput.emit(result);
    }
}
NgxfParseDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfParseDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
NgxfParseDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.0.1", type: NgxfParseDirective, selector: "[ngxf-parse]", inputs: { fileOption: ["ngxf-validate", "fileOption"], multiple: "multiple", accept: "accept", structure: "structure" }, outputs: { uploadOutput: "ngxf-parse" }, host: { listeners: { "paste": "parse($event)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfParseDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxf-parse]',
                }]
        }], propDecorators: { uploadOutput: [{
                type: Output,
                args: ['ngxf-parse']
            }], fileOption: [{
                type: Input,
                args: ['ngxf-validate']
            }], multiple: [{
                type: Input
            }], accept: [{
                type: Input
            }], structure: [{
                type: Input
            }], parse: [{
                type: HostListener,
                args: ['paste', ['$event']]
            }] } });

class NgxfUploaderService {
    constructor(http) {
        this.http = http;
    }
    /**
     * a method provide you to upload file through api easily.
     * @param option that upload detail options
     * @returns `UploadEvent` observable that combine with `status`, `percent`, `data`
     */
    upload(option) {
        const { files, method = 'POST', url, headers, params, process, filesKey, fields } = option, rest = __rest(option, ["files", "method", "url", "headers", "params", "process", "filesKey", "fields"]);
        if (files instanceof Blob ||
            files instanceof File ||
            (files instanceof Array && files.length !== 0)) {
            const uploadData = objectToFormData(option);
            const request = new HttpRequest(method, url, uploadData, Object.assign({ headers: headers instanceof HttpHeaders ? headers : new HttpHeaders(headers), params: params instanceof HttpParams
                    ? params
                    : new HttpParams({ fromObject: params }), reportProgress: process }, rest));
            return this.http.request(request).pipe(filter((response) => {
                if (process) {
                    return !(response instanceof HttpHeaderResponse ||
                        response.type === HttpEventType.DownloadProgress);
                }
                return response.type === HttpEventType.Response;
            }), map((event) => {
                switch (event.type) {
                    case HttpEventType.Sent:
                        return {
                            status: 0 /* Uploading */,
                            percent: 0,
                        };
                    case HttpEventType.UploadProgress:
                        return {
                            status: 0 /* Uploading */,
                            percent: Math.round((100 * event.loaded) / event.total) || 0,
                        };
                    case HttpEventType.Response:
                        return {
                            status: 1 /* Completed */,
                            percent: 100,
                            data: event.body,
                        };
                    default:
                        return {};
                }
            }), catchError((error) => {
                return throwError({
                    status: 2 /* UploadError */,
                    data: error.error,
                });
            }));
        }
        return throwError({ status: 3 /* FileNumError */ });
    }
}
NgxfUploaderService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderService, deps: [{ token: i1.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
NgxfUploaderService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }]; } });
const defaultFileKey = 'file';
function objectToFormData({ fields, files, filesKey }) {
    const data = new FormData();
    // * add fields
    if (fields) {
        Object.keys(fields).forEach((key) => data.append(key, fields[key]));
    }
    // * add files
    if (files instanceof Blob || files instanceof File) {
        data.append(filesKey || defaultFileKey, files, files['name'] || 'blob');
    }
    else {
        for (let i = 0; i < files.length; i++) {
            let key = defaultFileKey;
            // * if it has key
            if (filesKey) {
                if (filesKey instanceof Array) {
                    key = filesKey[i % filesKey.length];
                }
                else {
                    key = filesKey;
                }
            }
            data.append(key, files[i], files[i]['name'] || `blob${i}`);
        }
    }
    return data;
}

/**
 * ### ngxf-upload
 *
 * A file upload module provide you can upload file easily
 */
class NgxfUploaderModule {
}
NgxfUploaderModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxfUploaderModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderModule, declarations: [NgxfSelectDirective, NgxfDropDirective, NgxfParseDirective], exports: [NgxfSelectDirective, NgxfDropDirective, NgxfParseDirective] });
NgxfUploaderModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: NgxfUploaderModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [NgxfSelectDirective, NgxfDropDirective, NgxfParseDirective],
                    exports: [NgxfSelectDirective, NgxfDropDirective, NgxfParseDirective],
                }]
        }] });

/*
 * Public API Surface of ngxf-uploader
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxfDropDirective, NgxfParseDirective, NgxfSelectDirective, NgxfUploaderModule, NgxfUploaderService };
